Package: EHRtemporalVariability
Type: Package
Title: Delineating Temporal Dataset Shifts in Electronic Health Records
Version: 1.2.2
Date: 2026-04-27
Encoding: UTF-8
Description: Functions to delineate temporal dataset shifts in Electronic Health 
             Records through the projection and visualization of dissimilarities 
             among data temporal batches. This is done through the estimation of 
             data statistical distributions over time and their projection in 
             non-parametric statistical manifolds, uncovering the patterns of the 
             data latent temporal variability. 'EHRtemporalVariability' is 
             particularly suitable for multi-modal data and categorical variables 
             with a high number of values, common features of biomedical data where 
             traditional statistical process control or time-series methods may not 
             be appropriate. 'EHRtemporalVariability' allows you to explore and 
             identify dataset shifts through visual analytics formats such as 
             Data Temporal heatmaps and Information Geometric Temporal (IGT) plots. 
             An additional 'EHRtemporalVariability' Shiny app can be used to load 
             and explore the package results and even to allow the use of these 
             functions to those users non-experienced in R coding. (Sáez et al. 2020)
             <doi:10.1093/gigascience/giaa079>.
Depends: R (>= 4.3.0), dplyr
License: Apache License 2.0 | file LICENSE
Imports: plotly, zoo, xts, lubridate, RColorBrewer, viridis, scales,
        methods, MASS
Suggests: knitr, rmarkdown, devtools, dbscan, webshot, httr
VignetteBuilder: knitr
NeedsCompilation: no
Authors@R: c(
  person("Carlos", "Sáez", email = "carsaesi@upv.es", role = c("aut", "cre")),
  person("Alba", "Gutiérrez-Sacristán", email = "Alba_Gutierrez@hms.harvard.edu", role = c("aut")),
  person("Isaac", "Kohane", email = "Isaac_Kohane@hms.harvard.edu", role = c("aut")),
  person("Juan M", "García-Gómez", email = "juanmig@upv.es", role = c("aut")),
  person("Paul", "Avillach", email = "Paul_Avillach@hms.harvard.edu", role = c("aut")),
  person("Biomedical Data Science Lab, Universitat Politècnica de València (Spain)", role = c("cph")),
  person("Department of Biomedical Informatics, Harvard Medical School", role = c("cph"))
  )
Maintainer: Carlos Sáez <carsaesi@upv.es>
RoxygenNote: 7.3.3
URL: https://github.com/hms-dbmi/EHRtemporalVariability
Packaged: 2026-04-28 07:36:08 UTC; Carlos
Author: Carlos Sáez [aut, cre],
  Alba Gutiérrez-Sacristán [aut],
  Isaac Kohane [aut],
  Juan M García-Gómez [aut],
  Paul Avillach [aut],
  Biomedical Data Science Lab, Universitat Politècnica de València
    (Spain) [cph],
  Department of Biomedical Informatics, Harvard Medical School [cph]
Repository: CRAN
Date/Publication: 2026-04-28 08:10:17 UTC
Built: R 4.6.0; ; 2026-04-28 11:14:56 UTC; unix
